// RunCPM extensions by P.Z.
// DS3231 module connected via I2C bus

#ifndef DATETIME_H
#define DATETIME_H
#include "Wire.h"
#define DS3231_I2C_ADDRESS 0x68

// Convert normal decimal numbers to binary coded decimal
byte decToBcd(byte val)
{
	return((val / 10 * 16) + (val % 10));
}

// Convert binary coded decimal to normal decimal numbers
byte bcdToDec(byte val)
{
	return((val / 16 * 10) + (val % 16));
}

byte addrToDec (uint16 addr)
{
	return bcdToDec(_RamRead(addr));
}

// User date/time callback function.
void dateTime(uint16_t* date, uint16_t* time)
{
	uint16_t year;
	uint8_t month, day, hour, minute, second;

	Wire.begin();
	Wire.beginTransmission(DS3231_I2C_ADDRESS);
	Wire.write(0);  // set DS3231 register pointer to 00h
	Wire.endTransmission();
	Wire.requestFrom(DS3231_I2C_ADDRESS, 7);
	// request seven bytes of data from DS3231 starting from register 00h
	second = bcdToDec(Wire.read());
	minute = bcdToDec(Wire.read());
	hour   = bcdToDec(Wire.read());
	Wire.read(); // day of week - not used
	day    = bcdToDec(Wire.read());
	month  = bcdToDec(Wire.read());
	year   = bcdToDec(Wire.read()) + 2000;

	*date = FAT_DATE(year, month, day);
	*time = FAT_TIME(hour, minute, second);
}

// Get file date/time to DMA buffer
uint8 _GetFileStamp(uint16 fcbaddr, uint16 addr)
{
	CPM_FCB* F = (CPM_FCB*)_RamSysAddr(fcbaddr);
	uint8 result = 0xff;

	if (!_SelectDisk(F->dr)) {
		dir_t dir;
		_FCBtoHostname(fcbaddr, &filename[0]);
		File f = SD.open((char*)filename, O_READ);
		f.dirEntry(&dir);
		_RamWrite(addr,      decToBcd((FAT_YEAR(dir.creationDate) - 2000)));
		_RamWrite(addr +  1, decToBcd(FAT_MONTH(dir.creationDate)));
		_RamWrite(addr +  2, decToBcd(FAT_DAY(dir.creationDate)));
		_RamWrite(addr +  3, decToBcd(FAT_HOUR(dir.creationTime)));
		_RamWrite(addr +  4, decToBcd(FAT_MINUTE(dir.creationTime)));
		_RamWrite(addr +  5, decToBcd((FAT_YEAR(dir.lastAccessDate) - 2000)));
		_RamWrite(addr +  6, decToBcd(FAT_MONTH(dir.lastAccessDate)));
		_RamWrite(addr +  7, decToBcd(FAT_DAY(dir.lastAccessDate)));
		_RamWrite(addr +  8, 0); // dir.lastAccessTime doesn't exist
		_RamWrite(addr +  9, 0);
		_RamWrite(addr + 10, decToBcd((FAT_YEAR(dir.lastWriteDate) - 2000)));
		_RamWrite(addr + 11, decToBcd(FAT_MONTH(dir.lastWriteDate)));
		_RamWrite(addr + 12, decToBcd(FAT_DAY(dir.lastWriteDate)));
		_RamWrite(addr + 13, decToBcd(FAT_HOUR(dir.lastWriteTime)));
		_RamWrite(addr + 14, decToBcd(FAT_MINUTE(dir.lastWriteTime)));
		f.close();
		result = 0x01;
	}
	return(result);
}


// Set file date/time from DMA buffer
uint8 _SetFileStamp(uint16 fcbaddr, uint16 addr)
{
	CPM_FCB* F = (CPM_FCB*)_RamSysAddr(fcbaddr);
	uint8 result = 0xff;

	if (!_SelectDisk(F->dr)) {
		SdFile::dateTimeCallbackCancel();  // disable date/time callback function
		_FCBtoHostname(fcbaddr, &filename[0]);
		File f = SD.open((char*)filename, O_CREAT | O_WRITE);
		// set creation date time
		f.timestamp(T_CREATE,
			addrToDec(addr) + 2000,      // year
			addrToDec(addr + 1),         // month
			addrToDec(addr + 2),         // day
			addrToDec(addr + 3),         // hour
			addrToDec(addr + 4),         // minute
			0);                          // second - not used
		// set access date
		f.timestamp(T_ACCESS,
			addrToDec(addr + 5) + 2000,  // year
			addrToDec(addr + 6),         // month
			addrToDec(addr + 7),         // day
			addrToDec(addr + 8),         // hour
			addrToDec(addr + 9),         // minute
			0);                          // second - not used
		// set write/modification date time
		f.timestamp(T_WRITE,
			addrToDec(addr + 10) + 2000, // year
			addrToDec(addr + 11),        // month
			addrToDec(addr + 12),        // day
			addrToDec(addr + 13),        // hour
			addrToDec(addr + 14),        // minute
			0);                          // second - not used
		f.close();
		result = 0x01;
		SdFile::dateTimeCallback(dateTime); // enable date/time callback function
	}
	return(result);
}


// Get system date/time
void _GetTime(uint16 addr)
{
	Wire.beginTransmission(DS3231_I2C_ADDRESS);
	Wire.write(0);   // set DS3231 register pointer to 0
	Wire.endTransmission();
	// request seven bytes of data from DS3231 starting register 0
	Wire.requestFrom(DS3231_I2C_ADDRESS, 7);
	for (int i = 5; i >= 0; i--) {
		_RamWrite(addr + i, Wire.read());
		if (i == 3) {
			Wire.read(); // day of week - not used
		}
	}
}

// Set system date/time
void _SetTime(uint16 addr)
{
	Wire.beginTransmission(DS3231_I2C_ADDRESS);
	Wire.write(0);   // set DS3231 register pointer to 0
	// write bytes of data to DS3231 starting register 0
	for (int i = 5; i >= 0; i--) {
		Wire.write(_RamRead(addr + i));
		if (i == 3) {
			Wire.write(1); // day of week not used
		}
	}
	Wire.endTransmission();
}

#endif

